package st.tori.android.core.activity;

import java.io.IOException;
import java.util.Set;
import java.util.UUID;

import st.tori.android.core.util.DialogUtil;
import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.util.Log;

/*
 * 簡単に特定のBluetooth端末とペアリングする為のクラス
 */
public abstract class BluetoothActivity extends Activity {

	public static final String EXTRA_NAME_BT_DEVICE	 = "BluetoothActivity.btDevice";
	public static final String EXTRA_NAME_BT_SOCKET	 = "BluetoothActivity.btSocket";
	
	protected boolean suspendBluetooth = false;
	
	/*
	 * trueを返すと自動検出しない
	 */
    protected boolean isBluetoothInactivated(){
    	return false;
    }
	/*
	 * 手順1:onCreate内の適切なタイミングで下記を呼ぶ
	 */
    protected void onCreateBluetooth(Bundle savedInstanceState) {
    	Log.d(TAG,"onCreateBluetooth");
    	if(isBluetoothInactivated())return;
    	BluetoothDevice btDevice = getIntent().getParcelableExtra(EXTRA_NAME_BT_DEVICE);
    	if(btDevice!=null){
    		Log.d(TAG,"Reuse btDevice:btSocket="+btSocket);
    		this.btDevice = btDevice;
            IntentFilter filter = new IntentFilter(BluetoothDevice.ACTION_FOUND);
            this.getApplicationContext().registerReceiver(mReceiver, filter); // Don't forget to unregister during onDestroy
            onBluetoothConnectResult(RESULT_OK);
    	}else{
    		Log.d(TAG,"Search btDevice");
            IntentFilter filter = new IntentFilter(BluetoothDevice.ACTION_FOUND);
            this.getApplicationContext().registerReceiver(mReceiver, filter); // Don't forget to unregister during onDestroy
            setupBluetooth();
    	}
    }
	/*
	 * 手順2:他にstartActivityForResultを行う場合は、下記getRequestCodeEnableBT()がユニークとなるように注意する
	 */
	protected int getRequestCodeEnableBT(){	return 1;	}
	/*
	 * 手順3:onActivityResultをOverrideする際は、下記のように必ずsuperも呼ぶ
	 * super.onActivityResult(requestCode, resultCode, data);
	 */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data){
        if(getRequestCodeEnableBT()!=requestCode)return;
    	if(RESULT_OK==resultCode){
    		setupBluetooth();
    	}else{
    		DialogUtil.showSimpleDialog(this, "Bluetooth Required", "This application cannot be run without Bluetooth.");
    	}
    }

	/*
	 * 手順4:検出したいBluetooth機器のdeviceNameとpassKeyを指定する
	 */
	protected abstract String getBluetoothTargetDeviceName();
	protected abstract String getBluetoothTargetPassKey();
	/*
	 * 手順5:接続が確立または失敗したときの処理を定義する
	 */
    protected abstract void onBluetoothConnectResult(int result);
	
	protected BluetoothDevice btDevice;
	protected BluetoothSocket btSocket;
	protected BluetoothDevice getBluetoothDevice(){
		return this.btDevice;
	}
	/*
	 * 手順6:Bluetooth経由の入出力はBluetoothSocketを使用する
	 * @see st.tori.jpredict.levistone.bizlogic.ReadWriteThread
	 */
	protected final BluetoothSocket getBluetoothSocket(){
		openBluetoothSocketIfClosed();
		return btSocket;
	}
    
	private static final String TAG = "BluetoothActivity";
	public static final UUID SPP_UUID = UUID.fromString("00001101-0000-1000-8000-00805F9B34FB");
	
    // Create a BroadcastReceiver for ACTION_FOUND
    private final BroadcastReceiver mReceiver = new BroadcastReceiver() {
    	public void onReceive(Context context, Intent intent) {
	        Log.d(TAG,"BroadcastReceiver.onreceive1");
    		String action = intent.getAction();
	        Log.d(TAG,"BroadcastReceiver.onreceive2:action="+action);
            // When discovery finds a device
            if (BluetoothDevice.ACTION_FOUND.equals(action)) {
            	// Get the BluetoothDevice object from the Intent
            	BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
                // Add the name and address to an array adapter to show in a ListView
				Log.d(TAG,"FOUND:"+device.getName() + " " + device.getAddress());
				if(getBluetoothTargetDeviceName().equals(device.getName())){
					pair(device);
				}
            }
	        Log.d(TAG,"BroadcastReceiver.onreceive3");
        }
    };
    private void pair(final BluetoothDevice device) {
    	DialogInterface.OnClickListener buttonAction = new DialogInterface.OnClickListener(){
    		@Override
    		public void onClick(DialogInterface dialog, int arg) {
    			dialog.dismiss();
    			connect(device);
    		}
    	};
		DialogUtil.showSimpleDialog(this, "Ready to pair", "Use passkey "+getBluetoothTargetPassKey()+" to pair", "OK", buttonAction);
    }
    private void connect(final BluetoothDevice device) {
        Log.d(TAG,"connect");
    	this.btDevice = device;
    	openBluetoothSocketIfClosed();
    	closeBluetoothSocket(false);
    	onBluetoothConnectResult(RESULT_OK);
    }
    private long socketOpenedTimeInMillis = -1;
    protected void openBluetoothSocketIfClosed(){
    	//Log.d(TAG,"openBluetoothSocketIfClosed:btDevice="+btDevice);
    	if(suspendBluetooth||btDevice==null||btSocket!=null)return;
    	socketOpenedTimeInMillis = System.currentTimeMillis();    	
    	try {
    		btSocket = btDevice.createRfcommSocketToServiceRecord(SPP_UUID);
    		btSocket.connect();
    	} catch (IOException e) {
    		e.printStackTrace();
    		closeBluetoothSocket(true);
    		onBluetoothConnectResult(RESULT_CANCELED);
        }
    }
    protected void closeBluetoothSocket(boolean force){
    	//Log.d(TAG,"closeBluetoothSocket:btSocket="+btSocket);
    	if(btSocket==null)return;
    	if(!force&&System.currentTimeMillis()-socketOpenedTimeInMillis<30*1000)return;
		try{	btSocket.close();	}catch(Exception ex){	ex.printStackTrace();	}
    	btSocket = null;
    }
	
	@Override
    public void onDestroy() {
    	super.onDestroy();
        this.getApplicationContext().unregisterReceiver(mReceiver);
    }
    private void setupBluetooth(){
    	Log.d(TAG,"setupBluetooth");
    	BluetoothAdapter mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
    	if (mBluetoothAdapter == null) {
    		DialogUtil.showSimpleDialog(this, "Bluetooth Required", "This application cannot be run without Bluetooth.");
    	}else if (!mBluetoothAdapter.isEnabled()) {
    		Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
    		startActivityForResult(enableBtIntent, getRequestCodeEnableBT());
		}else{
			Set<BluetoothDevice> pairedDevices = mBluetoothAdapter.getBondedDevices();
			// If there are paired devices
			if (pairedDevices.size() > 0) {
				// Loop through paired devices
				for (BluetoothDevice device : pairedDevices) {
					Log.d(TAG,"BOUNDED:"+device.getName() + " " + device.getAddress());
					if(getBluetoothTargetDeviceName().equals(device.getName())){
						connect(device);
						return;
					}
			    }
			}
			boolean started = mBluetoothAdapter.startDiscovery();
			if(!started){
	    		DialogUtil.showSimpleDialog(this, "Error", "Cannot start Bluetooth discovery sequence.");
			}
		}
    }
}
